open System_defs
open Component_defs

(* On crée une fenêtre *)
let () = Global.init (Format.sprintf "game_canvas:%dx%d:r=presentvsync" 800 600)

let wall1 = Box.create "wall_top" 0 0 800 10 (Gfx.color 0 0 255 255)
let wall2 = Box.create "wall_bottom" 0 590 800 10 (Gfx.color 0 0 255 255)
let wall3 = Box.create "wall_left" 0 10 10 580 (Gfx.color 0 255 0 255)
let wall4 = Box.create "wall_right" 790 10 10 580 (Gfx.color 0 255 0 255)

let ball = Box.create "ball" 50 295 10 10 (Gfx.color 0 0 0 255)
let () = Move_system.register (ball :> movable)


let random_v b =
  let a = Random.float (Float.pi/.2.0) -. (Float.pi /. 4.0) in
  let v = Vector.{x = cos a; y = sin a} in
  let v = Vector.mult 5.0 (Vector.normalize v) in
  if b then v else Vector.{ v with x = -. v.x }
let () = Random.self_init ()
let () = ball # velocity # set Vector.zero

let player1 = Box.create "player1" 30 (300-50) 10 100 (Gfx.color 0 0 0 255)
let () = Move_system.register (player1 :> movable)

let player2 = Box.create "player2" 760 (300-50) 10 100 (Gfx.color 0 0 0 255)
let () = Move_system.register (player2 :> movable)


let init dt =
  Ecs.System.init_all dt;
  false


let table = Hashtbl.create 16
let has_key s = Hashtbl.mem table s
let set_key s= Hashtbl.replace table s ()
let unset_key s = Hashtbl.remove table s
let v_up = Vector. { x = 0.0; y = -5.0 }
let v_down = Vector.sub Vector.zero v_up

let update dt =
  let () = match Gfx.poll_event () with
      KeyDown s -> set_key s; Gfx.debug "%s\n%!"s;
    | KeyUp s -> unset_key s
    | _ -> ()
  in
  player1 # velocity # set Vector.zero;
  player2 # velocity # set Vector.zero;

  if has_key "e" then player1 # velocity # set v_up;
  if has_key "d" then player1 # velocity # set v_down;
  if has_key "u" then player2 # velocity # set v_up;
  if has_key "j" then player2 # velocity # set v_down;

  Ecs.System.update_all dt;
  let s = Global.scoring () in
  if s <> 0 then begin
    let x, v = if s = 2 then 50.0, random_v true else 740., random_v false in
    ball # pos # set Vector.{x; y = 295.0 };
    ball # velocity # set v;
    Global.set_scoring 0;
  end;
  true

let run () = 
  Gfx.main_loop init;
  Gfx.main_loop update
